import { SignerEntry } from '../Generated/web/org/xrpl/rpc/v1/common_pb';
export default class XrpSignerEntry {
    readonly account?: string | undefined;
    readonly signerWeight?: number | undefined;
    /**
     * Constructs an XrpSignerEntry from a SignerEntry protocol buffer.
     *
     * @param signerEntry a SignerEntry (protobuf object) whose field values will be used to construct an XrpSignerEntry
     * @return an XrpSignerEntry with its fields set via the analogous protobuf fields.
     * @see https://github.com/ripple/rippled/blob/f43aeda49c5362dc83c66507cae2ec71cfa7bfdf/src/ripple/proto/org/xrpl/rpc/v1/common.proto#L471
     */
    static from(signerEntry: SignerEntry): XrpSignerEntry | undefined;
    /**
     * @param account An XRP Ledger address whose signature contributes to the multi-signature.
     *                It does not need to be a funded address in the ledger.
     * @param signerWeight The weight of a signature from this signer. A multi-signature is only valid if the sum
     *                     weight of the signatures provided meets or exceeds the SignerList's SignerQuorum value.
     */
    private constructor();
}
