"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xrp_utils_1 = __importDefault(require("./xrp-utils"));
const transaction_status_1 = __importDefault(require("./transaction-status"));
const xrp_error_1 = __importStar(require("./xrp-error"));
const transaction_result_1 = __importDefault(require("./model/transaction-result"));
async function sleep(milliseconds) {
    return new Promise((resolve) => setTimeout(resolve, milliseconds));
}
/**
 * An XrpClient which blocks on `send` calls until the transaction has reached a deterministic state.
 */
class ReliableSubmissionXrpClient {
    constructor(decoratedClient, network) {
        this.decoratedClient = decoratedClient;
        this.network = network;
    }
    async getBalance(address) {
        return this.decoratedClient.getBalance(address);
    }
    async getPaymentStatus(transactionHash) {
        return this.decoratedClient.getPaymentStatus(transactionHash);
    }
    async send(amount, destination, sender) {
        return this.sendWithDetails({
            amount,
            destination,
            sender,
        });
    }
    async sendWithDetails(sendXrpDetails) {
        const { sender } = sendXrpDetails;
        const transactionHash = await this.decoratedClient.sendWithDetails(sendXrpDetails);
        await this.awaitFinalTransactionResult(transactionHash, sender);
        return transactionHash;
    }
    async getLatestValidatedLedgerSequence(address) {
        return this.decoratedClient.getLatestValidatedLedgerSequence(address);
    }
    async getRawTransactionStatus(transactionHash) {
        return this.decoratedClient.getRawTransactionStatus(transactionHash);
    }
    async accountExists(address) {
        return this.decoratedClient.accountExists(address);
    }
    async paymentHistory(address) {
        return this.decoratedClient.paymentHistory(address);
    }
    async getPayment(transactionHash) {
        return this.decoratedClient.getPayment(transactionHash);
    }
    async enableDepositAuth(wallet) {
        const result = await this.decoratedClient.enableDepositAuth(wallet);
        const transactionHash = result.hash;
        const rawTransactionStatus = await this.awaitFinalTransactionResult(transactionHash, wallet);
        const finalStatus = this.determineFinalResult(rawTransactionStatus);
        return new transaction_result_1.default(transactionHash, finalStatus, rawTransactionStatus.isValidated);
    }
    determineFinalResult(rawTransactionStatus) {
        var _a;
        // Return pending if the transaction is not validated.
        if (!rawTransactionStatus.isValidated) {
            return transaction_status_1.default.Pending;
        }
        else {
            const transactionStatus = ((_a = rawTransactionStatus.transactionStatusCode) === null || _a === void 0 ? void 0 : _a.startsWith('tes')) ? transaction_status_1.default.Succeeded
                : transaction_status_1.default.Failed;
            return transactionStatus;
        }
    }
    async awaitFinalTransactionResult(transactionHash, sender) {
        const ledgerCloseTimeMs = 4 * 1000;
        await sleep(ledgerCloseTimeMs);
        // Get transaction status.
        let rawTransactionStatus = await this.getRawTransactionStatus(transactionHash);
        const { lastLedgerSequence } = rawTransactionStatus;
        if (!lastLedgerSequence) {
            return Promise.reject(new Error('The transaction did not have a lastLedgerSequence field so transaction status cannot be reliably determined.'));
        }
        // Decode the sending address to a classic address for use in determining the last ledger sequence.
        // An invariant of `getLatestValidatedLedgerSequence` is that the given input address (1) exists when the method
        // is called and (2) is in a classic address form.
        //
        // The sending address should always exist, except in the case where it is deleted. A deletion would supersede the
        // transaction in flight, either by:
        // 1) Consuming the nonce sequence number of the transaction, which would effectively cancel the transaction
        // 2) Occur after the transaction has settled which is an unlikely enough case that we ignore it.
        //
        // This logic is brittle and should be replaced when we have an RPC that can give us this data.
        const classicAddress = xrp_utils_1.default.decodeXAddress(sender.getAddress());
        if (!classicAddress) {
            throw new xrp_error_1.default(xrp_error_1.XrpErrorType.Unknown, 'The source wallet reported an address which could not be decoded to a classic address');
        }
        const sourceClassicAddress = classicAddress.address;
        // Retrieve the latest ledger index.
        let latestLedgerSequence = await this.getLatestValidatedLedgerSequence(sourceClassicAddress);
        // Poll until the transaction is validated, or until the lastLedgerSequence has been passed.
        /*
         * In general, performing an await as part of each operation is an indication that the program is not taking full advantage of the parallelization benefits of async/await.
         * Usually, the code should be refactored to create all the promises at once, then get access to the results using Promise.all(). Otherwise, each successive operation will not start until the previous one has completed.
         * But here specifically, it is reasonable to await in a loop, because we need to wait for the ledger, and there is no good way to refactor this.
         * https://eslint.org/docs/rules/no-await-in-loop
         */
        /* eslint-disable no-await-in-loop */
        while (latestLedgerSequence <= lastLedgerSequence &&
            !rawTransactionStatus.isValidated) {
            await sleep(ledgerCloseTimeMs);
            // Update latestLedgerSequence and rawTransactionStatus
            latestLedgerSequence = await this.getLatestValidatedLedgerSequence(sourceClassicAddress);
            rawTransactionStatus = await this.getRawTransactionStatus(transactionHash);
        }
        /* eslint-enable no-await-in-loop */
        return rawTransactionStatus;
    }
}
exports.default = ReliableSubmissionXrpClient;
//# sourceMappingURL=reliable-submission-xrp-client.js.map